@@ -1,5 +1,5 @@
 /**
- * Copyright (C) 2006-2015 Henning Norén
+ * Copyright (C) 2006-2017 Henning Norén
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -102,6 +102,23 @@ parseWhiteSpace(FILE *file) {
   return ch;
 }
 
+static char
+parseWhiteSpaceOrComment(FILE *file) {
+  int ch;
+  do {
+    do {
+      ch = getc(file);
+    } while(isWhiteSpace(ch));
+    if(ch == '%') {
+      do {
+	ch = getc(file);
+      } while(!(ch >= 0x09 && ch <= 0x0d) && ch != EOF);
+      ch = getc(file);
+    }
+  } while(isWhiteSpace(ch) || ch == '%');
+  return ch;
+}
+
 static char*
 parseName(FILE *file) {
   int ch;
@@ -627,8 +644,8 @@ parseEncrypObject(FILE *file, EncData *e) {
 	    e->s_handler = s_handler;
 	    ff = true;
 	  }
-	  break;
 	}
+	break;
       case 'L':
 	if(isWord(file, "ength")) {
 	  int tmp_l = parseIntWithC(file,parseWhiteSpace(file));
@@ -720,7 +737,7 @@ parseEncrypObject(FILE *file, EncData *e) {
 
   if(!fe)
     e->encryptMetaData = true;
-  if(!fl)
+  if(!fl || e->length == 0)
     e->length = 40;
   if(!fv)
     e->version = 0;
@@ -738,10 +755,14 @@ parseEncrypObject(FILE *file, EncData *e) {
     }
     else {
       if(fu && fo) {
-	if(o_len != 32)
+	if(o_len != 32) {
 	  fprintf(stderr, "WARNING: O-String != 32 Bytes: %d\n", o_len);
-	if(u_len != 32)
+	  fo = false;
+	}
+	if(u_len != 32) {
 	  fprintf(stderr, "WARNING: U-String != 32 Bytes: %d\n", u_len);
+	  fu = false;
+	}
       }
       if (cf && aesv2) {
 	e->revision = 3;
@@ -776,7 +797,7 @@ findEncryptObject(FILE *file, const int e_pos, EncData *e) {
 	if(ch >= '0' && ch <= '9') {
 	  ch = parseWhiteSpace(file);
 	  if(ch == 'o' && getc(file) == 'b' && getc(file) == 'j' &&
-	     parseWhiteSpace(file) == '<' && getc(file) == '<') {
+	     parseWhiteSpaceOrComment(file) == '<' && getc(file) == '<') {
 	    return parseEncrypObject(file, e);
 	  }
 	}
